/***************************************************************************
 *   Copyright (C) 2010 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ombluetooth.h"
#include <QApplication>
#include <QDebug>
#include <QMap>

OMBluetooth::OMBluetooth(QWidget *parent)
 : QWidget(parent)
{
	IconBasePath = "/usr/share/pixmaps/openmoko-panel-plugin-qt/";
	activeIconFile = "bt_1.png";
	inactiveIconFile = "bt_0.png";
	discoverableIconFile = "bt_2.png";


	activeIcon = QIcon(IconBasePath + activeIconFile);
	inactiveIcon = QIcon(IconBasePath + inactiveIconFile);
	discoverableIcon = QIcon(IconBasePath + discoverableIconFile);

	StatusPanelName = QObject::tr("status:");
	DiscoverablePanelName = QObject::tr("is discoverable:");
	NamePanelName = QObject::tr("name:");
	AddressPanelName = QObject::tr("address:");
	
	Configuration = new OMConfig("openmoko-panel-plugin-qt-bt", 0 );


	MainLayout = new QVBoxLayout();
	ActivateLayout = new QHBoxLayout();
	VisibleLayout = new QHBoxLayout();

	InfoBox = new OMInfoBox(QObject::tr("Device"), this);
	InfoBox->addPanel(StatusPanelName, "");
	InfoBox->addPanel(NamePanelName, "");
	InfoBox->addPanel(AddressPanelName, "");
	InfoBox->addPanel(DiscoverablePanelName, "");
	MainLayout->addWidget( InfoBox );

	VisibleButton = new QPushButton(QObject::tr("set discoverable"));
	VisibleLayout->addWidget(VisibleButton);
	connect(VisibleButton,SIGNAL(clicked()),this,SLOT(VisibleButtonclicked()));

	InvisibleButton = new QPushButton(QObject::tr("set undiscoverable"));
	VisibleLayout->addWidget(InvisibleButton);
	connect(InvisibleButton,SIGNAL(clicked()),this,SLOT(InvisibleButtonclicked()));

	MainLayout->addLayout(VisibleLayout);

	ActivateButton = new QPushButton(QObject::tr("turn Bluetooth on"));
	ActivateLayout->addWidget(ActivateButton);
	connect(ActivateButton,SIGNAL(clicked()),this,SLOT(ActivateButtonclicked()));

	DeactivateButton = new QPushButton(QObject::tr("turn Bluetooth off"));
	ActivateLayout->addWidget(DeactivateButton);
	connect(DeactivateButton,SIGNAL(clicked()),this,SLOT(DeactivateButtonclicked()));

	MainLayout->addLayout(ActivateLayout);


	DialogButtons = new OMDialogButtons(OMFlags::QuitButton | OMFlags::CloseButton | OMFlags::ConfigButton, this);
	MainLayout->addWidget( DialogButtons );
	connect(DialogButtons, SIGNAL(clicked(QFlags< OMFlags :: Flag >)), this, SLOT(DialogButtonClicked(QFlags< OMFlags :: Flag >)));


	setLayout( MainLayout );


	SysTrayIcon = new QSystemTrayIcon( inactiveIcon, this );
	connect(SysTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	SysTrayIcon->show();

	systemBus = new QDBusConnection( QDBusConnection::connectToBus( QDBusConnection::SystemBus, "system" ) );
	connectToUsageInterface();
	connectToBluezManager();

	if (systemBus->connect("org.freesmartphone.ousaged", "/org/freesmartphone/Usage", "org.freesmartphone.Usage","ResourceChanged", this, SLOT(ResourceChanged(QString,bool)) )) qDebug() << "Signal ResourceChanged connected";
	
	connect(BluezManagerInterface,SIGNAL(DefaultAdapterChanged(QDBusObjectPath)),this,SLOT(connectToBluezAdapter(QDBusObjectPath)));


}


OMBluetooth::~OMBluetooth()
{
}

void OMBluetooth::iconActivated(QSystemTrayIcon::ActivationReason)
{
    showMaximized ();
}


void OMBluetooth::DeactivateButtonclicked()
{
	Configuration->runPreDisableScript();
	if (UsageInterface->isValid()) UsageInterface->call("ReleaseResource","Bluetooth");
}


void OMBluetooth::ActivateButtonclicked()
{
	Configuration->runPreEnableScript();
	if (UsageInterface->isValid()) UsageInterface->call("RequestResource","Bluetooth");

}

void OMBluetooth::VisibleButtonclicked()
{
		QList<QVariant> arg;
		arg << "Discoverable" << QVariant::fromValue(QDBusVariant(true));
		
		DiscoverMessage.setArguments( arg );
		
		BluezAdapterInterface->connection ().call(DiscoverMessage);
		

}

void OMBluetooth::InvisibleButtonclicked()
{
		QList<QVariant> arg;
		arg << "Discoverable" << QVariant::fromValue(QDBusVariant(false));
		
		DiscoverMessage.setArguments( arg );
		
		BluezAdapterInterface->connection ().call(DiscoverMessage);


}

void OMBluetooth::connectToUsageInterface()
{
	UsageInterface = new OMUsageInterface( this );
	
	if (UsageInterface->isValid()) {
		
		QDBusReply <bool> reply = UsageInterface->call("GetResourceState","Bluetooth");
		ResourceChanged("Bluetooth", reply.value());
		
		}
	else qDebug() << "could not connect to usage interface";
}

void OMBluetooth::ResourceChanged(QString name, bool state)
{
    if (name == "Bluetooth") {

	VisibleButton->setEnabled(state);
	InvisibleButton->setEnabled(state);

	if (state) {
		SysTrayIcon->setIcon( activeIcon );	
		InfoBox->changePanelValue( StatusPanelName, QObject::tr("active") );
		Configuration->runPostEnableScript();
		}

	else {

		SysTrayIcon->setIcon( inactiveIcon );		
		InfoBox->changePanelValue( StatusPanelName, QObject::tr("inactive") );
		Configuration->runPostDisableScript();


		}

	}

}


void OMBluetooth::connectToBluezManager()
{
    	BluezManagerInterface = new QDBusInterface( "org.bluez", "/", "org.bluez.Manager", *systemBus, this);


}


void OMBluetooth::connectToBluezAdapter(QDBusObjectPath path)
{
	
	
	qDebug() << "bluetooth object path is " + path.path();

	BluezAdapterInterface = new QDBusInterface("org.bluez", path.path(), "org.bluez.Adapter", *systemBus, this);

	DiscoverMessage = QDBusMessage::createMethodCall("org.bluez", BluezAdapterInterface->path(), "org.bluez.Adapter", "SetProperty");

	connect(BluezAdapterInterface,SIGNAL(PropertyChanged(QString, QDBusVariant)), this, SLOT(PropertyChanged(QString, QDBusVariant)));

	getProperties();
		
}




void OMBluetooth::getProperties()
{
	QDBusReply< QMap <QString, QVariant> > reply = BluezAdapterInterface->call("GetProperties");

	QString yesno;

	if (reply.value().value("Discoverable").toBool()) { 
		yesno = QObject::tr("yes");
		SysTrayIcon->setIcon( discoverableIcon );		
		}
	else {
		yesno = QObject::tr("no");
		QDBusReply <bool> reply = UsageInterface->call("GetResourceState","Bluetooth");
		ResourceChanged("Bluetooth", reply.value());
		}

	InfoBox->changePanelValue(DiscoverablePanelName, yesno);
	
	InfoBox->changePanelValue(NamePanelName, reply.value().value("Name").toString());
	InfoBox->changePanelValue(AddressPanelName, reply.value().value("Address").toString());
	
}


void OMBluetooth::PropertyChanged(QString key, QDBusVariant val)
{
    getProperties();
}


void OMBluetooth::DialogButtonClicked(QFlags<OMFlags::Flag> flags)
{
    if ( flags.testFlag(OMFlags::QuitButton) ) qApp->quit();
    if ( flags.testFlag(OMFlags::CloseButton) ) hide();
    if ( flags.testFlag(OMFlags::ConfigButton) ) Configuration->showMaximized();

}