/***************************************************************************
 *   Copyright (C) 2010 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "omgsm.h"
#include <QDebug>
#include <QApplication>
#include <math.h>

OMGSM::OMGSM(QWidget *parent)
 : QWidget(parent)
{
	IconBasePath = "/usr/share/pixmaps/openmoko-panel-plugin-qt/";
	IconFiles << "gsm_0.png" << "gsm_20.png" << "gsm_40.png" << "gsm_60.png" << "gsm_80.png" << "gsm_100.png";
	foreach( QString filename, IconFiles ) {
		Icons << QIcon(IconBasePath + filename);
		}

	GSMNumberPanelName = QObject::tr("number:");
	GSMIMEIPanelName = QObject::tr("imei:");
	GSMModelPanelName = QObject::tr("model:");
	GSMProviderPanelName = QObject::tr("provider:");
	GSMModePanelName = QObject::tr("mode:");
	GSMStrengthPanelName = QObject::tr("strength:");


	MainLayout = new QVBoxLayout();



//	TabWidget = new QTabWidget(this);

	GSMInfoBox = new OMInfoBox(QObject::tr("info"), this );
//	TabWidget->addTab(GSMInfoBox, QObject::tr("gsm"));

	GSMInfoBox->addPanel( GSMProviderPanelName, "" );
	GSMInfoBox->addPanel( GSMStrengthPanelName, "" );
	GSMInfoBox->addPanel( GSMNumberPanelName, "" );
	GSMInfoBox->addPanel( GSMIMEIPanelName, "" );
	GSMInfoBox->addPanel( GSMModelPanelName, "" );
	GSMInfoBox->addPanel( GSMModePanelName, "" );


	
//	GPRSInfoBox = new OMInfoBox(QObject::tr("info"), TabWidget );
//	TabWidget->addTab(GPRSInfoBox, QObject::tr("gprs"));

	MainLayout->addWidget( GSMInfoBox );

	SysTrayIcon = new QSystemTrayIcon( Icons.at( 0 ), this );
	connect(SysTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	SysTrayIcon->show();

	DialogButtons = new OMDialogButtons(OMFlags::QuitButton | OMFlags::CloseButton, this);
	MainLayout->addWidget( DialogButtons );
	connect(DialogButtons, SIGNAL(clicked(QFlags< OMFlags :: Flag >)), this, SLOT(DialogButtonClicked(QFlags< OMFlags :: Flag >)));

	setLayout( MainLayout );

	systemBus = new QDBusConnection( QDBusConnection::connectToBus( QDBusConnection::SystemBus, "system" ) );
    	connectToNetworkInterface();
	connectToDeviceInterface();
    	connectToSIMInterface();
	connectToUsageInterface();

	if (systemBus->connect("org.freesmartphone.ousaged", "/org/freesmartphone/Usage", "org.freesmartphone.Usage","ResourceChanged", this, SLOT(ResourceChanged(QString,bool)) )) qDebug() << "Signal ResourceChanged connected";

	if (systemBus->connect("org.freesmartphone.ogsmd", "/org/freesmartphone/GSM/Device", "org.freesmartphone.GSM.Network","Status", this, SLOT(NetworkStatusChanged() ))) qDebug() << "Signal Network Status connected";

}


OMGSM::~OMGSM()
{
}




void OMGSM::iconActivated(QSystemTrayIcon::ActivationReason)
{
    show();
}


void OMGSM::connectToDeviceInterface()
{
    DeviceInterface = new QDBusInterface("org.freesmartphone.ogsmd", "/org/freesmartphone/GSM/Device", "org.freesmartphone.GSM.Device", *systemBus, this);
}


void OMGSM::connectToSIMInterface()
{
    	SIMInterface = new QDBusInterface("org.freesmartphone.ogsmd", "/org/freesmartphone/GSM/Device", "org.freesmartphone.GSM.SIM", *systemBus, this);

	connect(SIMInterface,SIGNAL( ReadyStatus( bool ) ), this, SLOT( ReadyStatuschanged( bool ) ) );

	QDBusReply <bool> reply = SIMInterface->call("GetSimReady");

	if( reply ) getSIMInfo();
	
}

void OMGSM::connectToNetworkInterface()
{
    	NetworkInterface = new QDBusInterface("org.freesmartphone.ogsmd", "/org/freesmartphone/GSM/Device", "org.freesmartphone.GSM.Network", *systemBus, this);

//	connect(NetworkInterface,SIGNAL( ReadyStatus( bool ) ), this, SLOT( ReadyStatuschanged( bool ) ) );

}



void OMGSM::updateDeviceInfo()
{
	QDBusReply <QMap <QString,QVariant> > reply = DeviceInterface->call("GetInfo");
	GSMInfoBox->changePanelValue( GSMIMEIPanelName, reply.value().value("imei").toString() );
	GSMInfoBox->changePanelValue( GSMModelPanelName, reply.value().value("model").toString() );

}

void OMGSM::ReadyStatuschanged(bool status)
{
    if( status ) getSIMInfo();
	
}


void OMGSM::getSIMInfo()
{
 	QDBusReply< QMap <QString, QVariant> >reply = SIMInterface->call("GetSimInfo");

	QDBusArgument arg = qvariant_cast<QDBusArgument>(reply.value().value("subscriber_numbers"));

	QList <QString> list;
	arg.beginArray();
	arg >> list;
	arg.endArray(); 

	GSMInfoBox->changePanelValue( GSMNumberPanelName, list.at( 1 ) );	
}


void OMGSM::ResourceChanged(QString device, bool status)
{
    if ( device == "GSM" ) {
		if ( status) { 
			updateDeviceInfo();
			updateNetworkInfo();
		}
		else {
			SysTrayIcon->setIcon( Icons.at( 0 ) );
			GSMInfoBox->changePanelValue( GSMProviderPanelName, "" );
			GSMInfoBox->changePanelValue( GSMModePanelName, "" );
			GSMInfoBox->changePanelValue( GSMStrengthPanelName, "" );
		}
	}
}

void OMGSM::connectToUsageInterface()
{
	UsageInterface = new OMUsageInterface( this );
	
	QDBusReply <bool> reply = UsageInterface->call("GetResourceState","GSM");
	if ( reply ) ResourceChanged("GSM", reply.value());
		
}



void OMGSM::updateNetworkInfo()
{
    	QDBusReply <QMap <QString, QVariant> > reply = NetworkInterface->call( "GetStatus" );
	GSMInfoBox->changePanelValue( GSMProviderPanelName, reply.value().value("provider").toString() );
	GSMInfoBox->changePanelValue( GSMModePanelName, reply.value().value("mode").toString() );
	GSMInfoBox->changePanelValue( GSMStrengthPanelName, reply.value().value("strength").toString() );

	int strength = reply.value().value("strength").toInt();
	int tmp = rint(strength / 20);
	SysTrayIcon->setIcon( Icons.at( tmp ) );
}


void OMGSM::NetworkStatusChanged()
{
    updateNetworkInfo();
}

void OMGSM::DialogButtonClicked(QFlags<OMFlags::Flag> flags)
{
    if ( flags.testFlag(OMFlags::QuitButton) ) qApp->quit();
    if ( flags.testFlag(OMFlags::CloseButton) ) hide();
}